﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WpfDesktopEdition

Class Window1
  '============================================================================
  Const UseTileCache As Boolean = True
  '============================================================================

  Private Sub WpfMap1_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles WpfMap1.Loaded

    'Init WpfMap ZoomLevels, Extent and Background
    Call InitializeWpfMap()
    'Adds the WorldMap Overlay
    Call CreateWorldMapOverlay()

    'We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
    WpfMap1.Refresh()

  End Sub

  Private Sub InitializeWpfMap()

    'Set the Map Unit. The reason for setting it to DecimalDegrees is that is what the shapefile’s unit of measure is inherently in.
    WpfMap1.MapUnit = GeographyUnit.DecimalDegree
    'Disables PanZoomBar and Logo
    WpfMap1.MapTools.PanZoomBar.IsEnabled = False
    WpfMap1.MapTools.Logo.IsEnabled = False

    'Sets the maximun ZoomOut and extent
    WpfMap1.MaximumScale = 73000000
    WpfMap1.RestrictExtent = New RectangleShape(-3600, 89, 3600, -89)

    'Set a proper extent for the Map.  
    WpfMap1.CurrentExtent = New RectangleShape(-120, 89, 132, -68)

    'Sets the map background
    Dim MapBackground As New GeoSolidBrush(GeoColor.GeographicColors.DeepOcean)
    WpfMap1.Background = New SolidColorBrush(System.Windows.Media.Color.FromRgb(MapBackground.Color.RedComponent, MapBackground.Color.GreenComponent, MapBackground.Color.BlueComponent))
    WpfMap1.BackgroundOverlay.BackgroundBrush = MapBackground

  End Sub

  Private Sub CreateWorldMapOverlay()

    'Create the WorldMap LayerOverlay to holds the background chart
    Dim WorldMapOverlay As LayerOverlay = New LayerOverlay()
    WorldMapOverlay.Name = "WorldMapOverlay"
    WorldMapOverlay.WrappingMode = WrappingMode.WrapDateline
    WorldMapOverlay.TransitionEffect = TransitionEffect.None
    'WorldMapOverlay.TileType = TileType.MultipleTile

    'WorldMapOverlay.TileBuffer = 2
    WorldMapOverlay.WrappingExtent = New RectangleShape(-180, 90, 180, -90)

    'TileCache
    If UseTileCache Then
      Try
        Dim BitmapTileCache As New InMemoryBitmapTileCache()
        'BitmapTileCache.CacheDirectory = System.Windows.Forms.Application.StartupPath & "\Data\TilesCache"
        BitmapTileCache.CacheId = "WorldMapOverlay"
        BitmapTileCache.TileAccessMode = TileAccessMode.ReadAddDelete
        BitmapTileCache.ImageFormat = TileImageFormat.Png
        WorldMapOverlay.TileCache = BitmapTileCache
        WorldMapOverlay.ImageFormat = TileImageFormat.Png
      Catch ex As Exception
        'Call LOG("WorldMapOverlay TilesCache load error: " & ex.Message, 2, "MapView")
      End Try
    End If

    Dim WorldCountriesLayer As ShapeFileFeatureLayer = New ShapeFileFeatureLayer(System.Windows.Forms.Application.StartupPath & "\Data\WorldMap\Countries.shp", ShapeFileReadWriteMode.ReadOnly)
    WorldCountriesLayer.Name = "WorldCountriesLayer"
    WorldCountriesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County1
    WorldCountriesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    WorldCountriesLayer.WrappingExtent = New RectangleShape(-180, 90, 180, -90)
    WorldCountriesLayer.WrappingMode = WrappingMode.WrapDateline
    WorldMapOverlay.Layers.Add("WorldCountriesLayer", WorldCountriesLayer)

    'And finally add the layerOverlay to the map.
    WpfMap1.Overlays.Add("WorldOverlay", WorldMapOverlay)

  End Sub

  Dim TestPositions As New Stack(Of PointShape)
  Dim TestRunning As Boolean = False

  Private Sub btnTest_Click(sender As Object, e As RoutedEventArgs) Handles btnTest.Click

    Debug.WriteLine("Test Started")
    TestRunning = True

    'Generate Test Points
    For t As Integer = 1 To 10
      For value As Integer = 1 To 10
        TestPositions.Push(New PointShape(value, value))
      Next
    Next

    'ZoomTo first position 0,0 Scale 100,000
    WpfMap1.ZoomTo(New PointShape(0, 0), 100000)

  End Sub

  Dim stopWatch As New System.Diagnostics.Stopwatch()

  Private Sub WpfMap1_OverlayDrawing(sender As Object, e As OverlayDrawingWpfMapEventArgs) Handles WpfMap1.OverlayDrawing
    stopWatch.Start()
  End Sub

  Private Sub WpfMap1_OverlayDrawn(sender As Object, e As OverlayDrawnWpfMapEventArgs) Handles WpfMap1.OverlayDrawn
    'Debug.WriteLine("Overlay drawn in " & stopWatch.Elapsed.TotalMilliseconds.ToString & "ms")
    Debug.WriteLine(stopWatch.Elapsed.TotalMilliseconds.ToString)
    stopWatch.Reset()

    'During test load next position
    If TestRunning Then
      If TestPositions.Count > 0 Then
        WpfMap1.ZoomTo(TestPositions.Pop(), 100000) 'Not displayed, but it works despite it doesn't seems to
      Else
        Debug.WriteLine("Test finished")
        TestRunning = False
        WpfMap1.ZoomToScale(72000000)
      End If
    End If

  End Sub
End Class
